'use strict';
angular.module('aom.services')
  .factory('aomComponentsSrv', function ($cordovaLaunchNavigator, $timeout, $q, $sce, aomLogSrv, aomPopupSrv, aomResourcesSrv, constants) {
    var components = {};

    components.data = {
      originalOrientation: null,
      oriantaionChanging: false
    };

    function isTomTom() {
      const _manufacturer = window.cordova ? window.device.manufacturer : window.navigator.vendor;
      return _manufacturer.toLowerCase().indexOf('tomtom') !== -1
        || _manufacturer.toLowerCase().indexOf('webfleet') !== -1;
    }

    function mapLaunchError(err) {
      aomPopupSrv.alert('MapLaunchFailed', {
        title: aomResourcesSrv.data.resources.Common.Error,
        template: aomResourcesSrv.data.resources.Errors.MapNavigatorLaunchFailed,
        onDismiss: function () { }
      });
      aomLogSrv.log({ message: 'Error in launching navigator', error: err }, aomLogSrv.levels.high, aomLogSrv.severities.err);
    }

    components.markerColourCurrentLocation = L.AwesomeMarkers.icon({ icon: 'coffee', markerColor: 'red', draggable: false });

    components.generateUniqueId = function () {
      var text = '';
      var possible = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';

      for (var i = 0; i < 15; i++) {
        text += possible.charAt(Math.floor(Math.random() * possible.length));
      }
      return text;
    };

    components.launchNavigator = function (destinationLatitude, destinationLongitude) {
      var destination = [destinationLatitude, destinationLongitude];
      if (window.cordova) {
        if (isTomTom()) {
          window.plugins.tomTomBridgeNavigationLauncher.navigate(destination, null, mapLaunchError);
        } else {
          $cordovaLaunchNavigator.navigate(destination).catch(mapLaunchError);
        }
      } else {
        alert('Ionic Serve wont do navigation for destination: ' + destination);
      }
    };

    components.lockOrientation = function () {
      if (window.cordova) {
        var isPortrait = screen.orientation.type !== 'landscape-primary' && screen.orientation.type !== 'landscape-secondary';
        var orientation = 'landscape-primary';
        if (!isPortrait) {
          if (ionic.Platform.isIOS()) {
            orientation = screen.orientation.type === 'landscape-primary' ? 'landscape-secondary' : 'landscape-primary';
          } else {
            orientation = screen.orientation.type;
          }
        }
        return screen.orientation.lock(orientation);
      } else {
        return $q.when();
      }
    };

    components.unlockOrientation = function () {
      if (window.cordova) {
        return screen.orientation.unlock();
      } else {
        return $q.when();
      }
    };

    components.showKeyboard = function () {
      if (window.cordova && ionic.Platform.isEdge()) {
        $timeout(function () {
          cordova.plugins.Keyboard.show();
        });
      }
    };

    components.canPrintImages = function () {
      var canPrint = true;
      if (ionic.Platform.isEdge()) {
        canPrint = Windows.System.Profile.AnalyticsInfo.versionInfo.deviceFamily === 'Windows.Desktop'; //https://gist.github.com/wagonli/40d8a31bd0d6f0dd7a5d
      }
      return canPrint;
    };

    components.makeTrustedURL = function (filepath) {
      return $sce.trustAsResourceUrl(filepath + '#t=1');
    };

    components.dataURLtoBlob = function (dataurl) {
      var arr = dataurl.split(','),
        mime = arr[0].match(/:(.*?);/)[1],
        bstr = atob(arr[1]),
        n = bstr.length,
        u8arr = new Uint8Array(n);
      while (n--) {
        u8arr[n] = bstr.charCodeAt(n);
      }
      return new Blob([u8arr], { type: mime });
    };

    components.getLeafletCurrentLocationControl = function getLeafletCurrentLocationButton(event) {
      L.Control.CurrentLocation = L.Control.extend({
        onAdd: function () {
          var button = L.DomUtil.create('button');
          L.DomUtil.addClass(button, 'button');
          button.style.padding = '0px';
          button.style['background-color'] = 'transparent';
          button.style['margin-left'] = '0px';
          button.innerHTML = '<i style="font-size:40px" class="far fa-location"></i>';

          L.DomEvent.on(button, 'click', event);
          return button;
        }
      });

      return new L.Control.CurrentLocation({ position: 'topleft' });
    };

    components.getExtensionForContentType = contentType => {
      return Object.keys(constants.extensionsToMimeTypes).filter(key => constants.extensionsToMimeTypes[key] === contentType)[0];
    };

    components.databasePath = function databasePath() {
      if (ionic.Platform.isIOS()) {
        return cordova.file.applicationStorageDirectory + 'Library/LocalDatabase/';
      } else if (ionic.Platform.isAndroid()) {
        return cordova.file.applicationStorageDirectory + 'databases/';
      }
    };

    return components;
  });