'use strict';
angular.module('aom.services')
  .factory('aomFileSystemSrv', function ($q, $cordovaFile, $cordovaFileError, $ionicPlatform, constants, aomResourcesSrv, aomLogSrv, aomPopupSrv) {
    var fileSystem = {};

    fileSystem.data = {
      selectedJobId: ''
    };

    function path(folder) {
      return cordova.file.dataDirectory + folder + '/';
    }

    function removeFileIfExists(path, filename) {
      var deferred = $q.defer();
      $cordovaFile.removeFile(path, filename)
        .finally(function () {
          deferred.resolve();
        });

      return deferred.promise;
    }

    function openSystemViewer(path, filename, mimeType, mediaPath) {
      removeFileIfExists(path, filename)
        .then(function () {
          $cordovaFile.copyFile(mediaPath, filename, path)
            .then(function (file) {
              cordova.plugins.fileOpener2.open(
                file.toURL(),
                mimeType,
                {
                  error: function (err) {
                    aomPopupSrv.alert('SystemViewerLaunchFailed', {
                      title: aomResourcesSrv.data.resources.Common.Error,
                      template: aomResourcesSrv.data.resources.Errors.SystemViewerLaunchFailed,
                      onDismiss: function () { }
                    });
                    aomLogSrv.log({ message: 'Cannot open system viewer', error: err }, aomLogSrv.levels.high, aomLogSrv.severities.err);
                  },
                  success: function () {

                  }
                }
              );
            }, function (err) {
              aomPopupSrv.alert('SystemViewerLaunchFailed', {
                title: aomResourcesSrv.data.resources.Common.Error,
                template: aomResourcesSrv.data.resources.Errors.SystemViewerLaunchFailed,
                onDismiss: function () { }
              });
              aomLogSrv.log({ message: 'Cannot open system viewer', error: err }, aomLogSrv.levels.high, aomLogSrv.severities.err);
            });
        });
    }

    function windowsEscapedPath(aPath) {
      return aPath.replace(/\//g, '\\');
    }

    function openWindowsSystemViewer(filename, mediaPath) {
      var applicationData = Windows.Storage.ApplicationData.current;
      var localFolder = applicationData.localFolder;
      var escapedPath = mediaPath ? windowsEscapedPath(mediaPath) : constants.common.aomAttachmentsDirectory;

      localFolder.getFolderAsync(escapedPath)
        .done(function (attachmentsFolder) {
          attachmentsFolder.getFileAsync(filename)
            .done(function (file) {
              Windows.System.Launcher.launchFileAsync(file);
            }, function (err) {
              aomPopupSrv.alert('SystemViewerLaunchFailed', {
                title: aomResourcesSrv.data.resources.Common.Error,
                template: aomResourcesSrv.data.resources.Errors.SystemViewerLaunchFailed,
                onDismiss: function () { }
              });
              aomLogSrv.log({ message: 'Cannot open system viewer', error: err }, aomLogSrv.levels.high, aomLogSrv.severities.err);
            });
        }, function (err) {
          aomPopupSrv.alert('SystemViewerLaunchFailed', {
            title: aomResourcesSrv.data.resources.Common.Error,
            template: aomResourcesSrv.data.resources.Errors.SystemViewerLaunchFailed,
            onDismiss: function () { }
          });
          aomLogSrv.log({ message: 'Cannot open system viewer', error: err }, aomLogSrv.levels.high, aomLogSrv.severities.err);
        });
    }

    window.Ionic = window.Ionic || {};

    fileSystem.normalizeURL = function (url) {
      if (!window.cordova || !window.Ionic.WebView.convertFileSrc) {
        return url;
      } else {
        return window.Ionic.WebView.convertFileSrc(url);
      }
    };

    fileSystem.copyToLocalData = function (location, name, newName) {
      var folder = constants.common.aomAttachmentsDirectory;
      var folderPath = path(folder);
      return $cordovaFile.copyFile(location, name, folderPath, newName);
    };

    fileSystem.fileExists = function (filename, folder) {
      folder = folder === undefined ? constants.common.aomAttachmentsDirectory : folder;
      var folderPath = path(folder);
      return $cordovaFile.checkFile(folderPath, filename);
    };

    fileSystem.logoExists = function (filename) {
      return fileSystem.listFilenames(path(constants.common.logoFolder))
        .then(files => {
          const logo = files.filter(file => file.name.indexOf(filename) !== -1);
          return logo.length > 0 ? $q.when(logo[0]) : $q.reject();
        });
    };

    fileSystem.write = function (filename, file, folder, existing) {
      if (window.cordova) {
        folder = folder === undefined ? constants.common.aomAttachmentsDirectory : folder;
        var folderPath = path(folder);
        var failMessage = 'Failed to write file ' + filename;
        if (existing) {
          return $cordovaFile.writeExistingFile(folderPath, filename, file.data)
            .catch(function (err) {
              aomLogSrv.log({ message: failMessage, error: err }, aomLogSrv.levels.medium, aomLogSrv.severities.err);
              return $q.reject(failMessage);
            });
        } else {
          return $cordovaFile.writeFile(folderPath, filename, file.data, true)
            .catch(function (err) {
              aomLogSrv.log({ message: failMessage, error: err }, aomLogSrv.levels.medium, aomLogSrv.severities.err);
              return $q.reject(failMessage);
            });
        }
      } else {
        return $q.when();
      }
    };

    fileSystem.applicationDataPath = window.cordova ? cordova.file.dataDirectory + constants.common.aomAttachmentsDirectory : '';
    fileSystem.rootApplicationPath = window.cordova ? cordova.file.dataDirectory : '';
    fileSystem.applicationDataPathNormalized = fileSystem.normalizeURL(fileSystem.applicationDataPath);
    fileSystem.adhocDocumentsPathNormalized = window.cordova ? fileSystem.normalizeURL(cordova.file.dataDirectory) + constants.common.aomAdhocAttachmentsDirectory : '';
    fileSystem.jobDataPathNormalized = fileSystem.applicationDataPathNormalized + '/';

    fileSystem.openSystemViewer = function (filename, mimeType, relativeToRootPath) {
      // Works only for opening job attachments
      var mediaPath = relativeToRootPath ? fileSystem.rootApplicationPath + relativeToRootPath : fileSystem.applicationDataPath;
      if (ionic.Platform.isAndroid()) {
        openSystemViewer(cordova.file.externalCacheDirectory, filename, mimeType, mediaPath);
      } else if (ionic.Platform.isIOS()) {
        openSystemViewer(cordova.file.cacheDirectory, filename, mimeType, mediaPath);
      } else if (ionic.Platform.isEdge()) {
        openWindowsSystemViewer(filename, relativeToRootPath);
      }
    };

    fileSystem.removeFolder = function (folder) {
      if (window.cordova) {
        return $cordovaFile.removeRecursively(cordova.file.dataDirectory, folder)
          .catch(function () {
            return $q.when();
          });
      } else {
        return $q.when();
      }
    };

    fileSystem.removeAttachment = function (file) {
      if (window.cordova) {
        return $cordovaFile.removeFile(fileSystem.applicationDataPath, file);
      } else {
        return $q.when();
      }
    };

    fileSystem.removeFile = function removeFile(path, filename) {
      return removeFileIfExists(path, filename);
    };

    fileSystem.removeDirectory = function removeDirectory(path, folder) {
      return $cordovaFile.removeRecursively(path, folder)
        .finally(function () {
          return $q.when();
        });
    };

    fileSystem.createDirectory = function (name) {
      var deferred = $q.defer();

      if (window.cordova) {
        $cordovaFile.createDir(cordova.file.dataDirectory, name, false)
          .finally(function () {
            deferred.resolve();
          });
      } else {
        deferred.resolve();
      }

      return deferred.promise;
    };


    fileSystem.listFilenames = function (path) {
      var q = $q.defer();

      try {
        window.resolveLocalFileSystemURL(path,
          function (fileSystem) {
            var reader = fileSystem.createReader();
            reader.readEntries(
              function (entries) {
                if (angular.isArray(entries)) {
                  q.resolve(entries);
                } else {
                  q.reject('Error returning non array data');
                }
              },
              function (error) {
                error.message = $cordovaFileError[error.code];
                q.reject(error);
              }
            );
          }, function (error) {
            error.message = $cordovaFileError[error.code];
            q.reject(error);
          }
        );
      } catch (error) {
        error.message = $cordovaFileError[error.code];
        q.reject(error);
      }

      return q.promise;
    };

    fileSystem.base64String = function (filename) {
      return $cordovaFile.readAsDataURL(fileSystem.applicationDataPath, filename);
    };

    return fileSystem;
  });

