﻿'use strict';
angular.module('aom.services')
  .factory('aomJobSrv', function ($rootScope, $ionicPopup, $q, $timeout, $state, aomSqlMapSrv, aomResourcesSrv, leafletData, aomLocationSrv, aomDatabaseSrv, aomFileSystemSrv, aomComponentsSrv, aomMarkersSrv, constants, aomPopupSrv, aomNotificationSrv, aomAutoSaveSrv, aomBackSrv, aomNetworkSrv, aomLogSrv) {
    var aomJobSrv = {};

    aomJobSrv.data = {
      selectedJob: {},
      selectedField: {},
      header: {
        title: ''
      },
      changesMade: false,
      initComplete: false,
      aomSqlMapSrvData: aomSqlMapSrv.data,
      showSubmitOption: null
    };

    aomJobSrv.fieldTypes = constants.fieldTypes;
    aomJobSrv.workLocationSentFromServer = false;

    function getDiffFields() {
      aomJobSrv.data.selectedJob
        .Fields
        .map(function (field) {
          if (field.Type === constants.fieldTypes.dateTime) field.DiffCallbacks = [];
          return field;
        })
        .filter(function (field) {
          return field.Type === constants.fieldTypes.dateTimeDiff;
        })
        .forEach(function (field) {
          var dtFields = field.Options.Dependencies.map(function (dtFieldRef) {
            return aomJobSrv.data.selectedJob.Fields[aomJobSrv.data.selectedJob.FieldsIndex[dtFieldRef]];
          });

          dtFields.forEach(function (dtField) {
            dtField.DiffCallbacks.push(function () {
              if (dtFields[0].Data !== null && dtFields[0].Data !== '' && dtFields[1].Data !== null && dtFields[1].Data !== '') {
                dtFields[0].Data.setMilliseconds(0);
                dtFields[1].Data.setMilliseconds(0);

                var dateA = moment(dtFields[0].Data), dateB = moment(dtFields[1].Data);

                var milliseconds = Math.abs(dateA.diff(dateB));
                var seconds = (milliseconds / 1000) % 60;
                var minutes = Math.floor((milliseconds / (1000 * 60)) % 60);
                var hours = Math.floor((milliseconds / (1000 * 60 * 60)));

                field.Data = hours + ':' + ('0' + minutes).slice(-2) + ':' + ('0' + seconds).slice(-2);
              }
            });
          });
        });
    }

    function setTitle() {
      aomJobSrv.data.header.title = aomJobSrv.data.selectedJob.Title;
    }

    aomJobSrv.setShowSubmitFlags = function () {
      if (aomJobSrv.data.selectedJob.HasSubmitButton) {
        aomJobSrv.data.showSubmitOption = Enumerable
          .from(aomJobSrv.data.selectedJob.Paging.Pages)
          .where('$.Name === "' + aomJobSrv.data.selectedJob.Paging.selectedPage + '"').toArray()[0].HasSubmitButton;
      } else {
        aomJobSrv.data.showSubmitOption = true;
      }
    };

    function tryRecovery() {
      aomJobSrv.data.initComplete = false;
      aomAutoSaveSrv.data.selectedJob = aomJobSrv.data.selectedJob;
      return aomAutoSaveSrv.handleRecovered()
        .then(function () {
          aomJobSrv.setupJob();
          aomSqlMapSrv.runIndependantSql(aomJobSrv.data.selectedJob);
          return $q.when();
        });
    }

    aomJobSrv.addMarker = aomMarkersSrv.addMarker;

    aomJobSrv.setupJob = function () {
      aomJobSrv.calc = aomJobSrv.data.features.runSql ? aomSqlMapSrv.calc : angular.noop;
      aomMarkersSrv.regenerateMarkers(aomJobSrv.data.selectedJob);
      aomJobSrv.data.changesMade = false;
      getDiffFields();
      if (aomJobSrv.data.selectedJob.isTemplate) {
        aomJobSrv.data.selectedJob.newId = aomComponentsSrv.generateUniqueId();
        aomJobSrv.data.selectedJob.DateCreated = moment().format();
      }
      aomFileSystemSrv.data.selectedJobId = aomJobSrv.data.selectedJob.Id;
      setTitle();
      aomJobSrv.setShowSubmitFlags();
      aomJobSrv.data.selectedFieldPosition = null;
      aomJobSrv.data.selectedField = {};
      aomJobSrv.data.downloadedAdhocAttachment = false;
      aomJobSrv.data.initComplete = true;
    };

    aomJobSrv.setBounds = function () {
      if (aomJobSrv.data.selectedJob.markerBounds.length !== 0) {
        leafletData.getMap('jobMap').then(function (map) {
          aomLocationSrv.getAccurateCurrentLocation()
            .then(function () {
              var currentLocationBound = [];
              currentLocationBound.push([aomLocationSrv.data.lastKnownLocation.latitude, aomLocationSrv.data.lastKnownLocation.longitude]);
              currentLocationBound.push(aomJobSrv.data.selectedJob.markerBounds);
              map.fitBounds(currentLocationBound, { maxZoom: 16, padding: [50, 50] });
            });
        });
      }
    };

    aomJobSrv.calculateBounds = function () {
      for (var i = 0; i < aomJobSrv.data.selectedJob.markersOnJob.length; i++) {
        aomJobSrv.data.selectedJob.markerBounds.push([aomJobSrv.data.selectedJob.markersOnJob[i].lat, aomJobSrv.data.selectedJob.markersOnJob[i].lng]);
      }
    };

    aomJobSrv.init = function () {
      if (aomJobSrv.data.selectedJob.readyToSubmit) {
        aomNotificationSrv.notify(aomResourcesSrv.data.resources.Job.JobHasBeenSubmitted);
      } else {
        return tryRecovery()
          .then(function () {
            aomJobSrv.data.selectedJob.jsProperties.status = constants.status.opened;
            return $q.when();
          })
          .catch(function (err) {
            aomLogSrv.log({ message: 'Error checking for recovered job', error: err }, aomLogSrv.levels.high, aomLogSrv.severities.err);
          });
      }
    };

    aomJobSrv.revert = function () {
      aomJobSrv.data.initComplete = false;
      aomAutoSaveSrv.data.selectedJob = aomJobSrv.data.selectedJob;
      return aomAutoSaveSrv.discardJob()
        .then(function () {
          return aomAutoSaveSrv.rollbackAttachmentUpdates(aomJobSrv.data.selectedJob.Id);
        })
        .then(function () {
          aomJobSrv.setupJob();
          aomSqlMapSrv.runIndependantSql(aomJobSrv.data.selectedJob);
          aomJobSrv.data.selectedJob.jsProperties.status = constants.status.opened;
          return $q.when();
        });
    };

    aomJobSrv.setField = function (field) {
      aomJobSrv.data.selectedField = field;
    };

    aomJobSrv.reset = function () {
      aomJobSrv.data.initComplete = false;
      aomJobSrv.setupJob();
      aomBackSrv.clear();
      $timeout(function () {
        $state.go('app.Job');
      });
    };

    aomJobSrv.showMap = function () {
      $timeout(function () {
        if (aomNetworkSrv.online()) {
          aomBackSrv.exit(false);
          $state.go('app.JobMap');
        } else {
          aomNotificationSrv.notify(aomResourcesSrv.data.resources.Errors.NoInternet);
        }
      }, 30);
    };

    return aomJobSrv;
  });
