﻿'use strict';
angular.module('aom.services')
  .factory('aomJobsSrv', function ($q, $rootScope, $state, $timeout, $filter, aomAPISrv, aomDatabaseSrv, aomCleanSrv, leafletData, aomJobDistanceSrv, aomLocationSrv, aomResourcesSrv, aomNetworkSrv, aomJobSrv, aomNotificationSrv, aomAttachmentsSrv, aomJobUpdatedSrv, aomBackSrv, aomPopupSrv, aomLogSrv, aomLoginTaskManagerSrv, aomOfflineDatabasesSrv, constants, aomEventSrv, aomHardwareSrv, aomTtNotificationSrv) {
    var aomJobsSrv = {};
    var setupTimeout = null;

    aomJobsSrv.data = {
      searchQuery: '',
      setSearchQuery: '',
      view: 'List',
      markers: [],
      bounds: [],
      templates: [],
      jobs: [],
      drafts: [],
      filteredJobs: [],
      filteredTemplates: [],
      currentApplicationCode: '',
      showUpgradePrompt: true,
      canShowAnimation: true
    };

    function updateDbForConfirmReceipt(jobOrTemplate) {
      var query = {},
        data = {
          id: jobOrTemplate.Id,
          detail_json: jobOrTemplate
        };
      jobOrTemplate.isTemplate ? query.Templates = data : query.Jobs = data;
      return aomDatabaseSrv.insertQuery(query, true, false);
    }

    function confirmReceipt() {
      var confirmJobs = Enumerable.from(aomJobsSrv.data.jobs).where('$.confirmedReceived === false').toArray();
      var confirmTemplates = Enumerable.from(aomJobsSrv.data.templates).where('$.confirmedReceived === false').toArray();
      var toConfirm = confirmJobs.concat(confirmTemplates);
      var confirmObject = { JobIds: Enumerable.from(confirmJobs.concat(confirmTemplates)).select('parseInt($.Id)').toArray() };
      var promises = [];

      if (toConfirm.length === 0) {
        return $q.when();
      } else {
        return aomAPISrv.postData(aomAPISrv.data.rootLinks.JobsAck.href, confirmObject)
          .then(function () {
            for (var i = 0; i < toConfirm.length; i++) {
              toConfirm[i].confirmedReceived = true;
              (function (i) {
                $timeout(function () {
                  promises.push(updateDbForConfirmReceipt(toConfirm[i]));
                }, i * 50);
              })(i);
            }

            if (aomResourcesSrv.data.resources.appConfig.ShowOptionalNotifications) {
              if (aomHardwareSrv.isTomTom()) {
                aomTtNotificationSrv.show(aomResourcesSrv.data.resources.Job.JobUpdateNotificationMessage);
              } else {
                aomNotificationSrv.notify(aomResourcesSrv.data.resources.Job.JobUpdateNotificationMessage, aomResourcesSrv.data.resources.appConfig.PersistentNotification);
              }
            }

            return $q.all(promises);
          })
          .catch(function (err) {
            aomLogSrv.log({ message: 'Failed in aomJobsSrv.confirmReceipt', error: err }, aomLogSrv.levels.medium, aomLogSrv.severities.warn);
          })
          .finally(function () {
            return $q.when();
          });
      }
    }

    function deleteJob(job) {
      var query = {},
        inMemoryData;
      if (job.isTemplate) {
        query = { Templates: { id: job.Id } };
        inMemoryData = aomJobsSrv.data.templates;
      } else {
        query = { Jobs: { id: job.Id } };
        inMemoryData = aomJobsSrv.data.jobs;
      }
      var removeIndex = inMemoryData.map(function (item) { return item.Id; })
        .indexOf(job.Id);

      var inMemoryJob = inMemoryData[removeIndex];

      if (aomJobUpdatedSrv.shouldShowJobDeletedUpdatedDialog(job.Id)) {
        if (inMemoryJob.isTemplate) {
          return $q.when();
        } else {
          aomJobUpdatedSrv.showJobDeletedDialog();
        }
      }
      if (inMemoryJob.jsProperties.status === constants.status.opened) {
        inMemoryJob.jsProperties.status = constants.status.deleted;
        inMemoryJob.readyToSubmit = true;
        return $q.when();
      } else {
        return aomAttachmentsSrv.deleteAttachmentFromDatabaseByJobId(job.Id)
          .then(function () {
            return aomDatabaseSrv.deleteQuery(query);
          })
          .then(function () {
            var newRemoveIndex = inMemoryData.map(function (item) { return item.Id; })
              .indexOf(job.Id);
            ~newRemoveIndex && inMemoryData.splice(newRemoveIndex, 1);
            setupJobsWithTimeout();
          })
          .catch(function (err) {
            aomLogSrv.log({ message: 'Failed in aomJobsSrv.deleteJob', error: err }, aomLogSrv.levels.medium, aomLogSrv.severities.warn);
          });
      }
    }

    function fetchAttachments(job) {
      return aomCleanSrv.getAttachments(job, job.links.attachments)
        .then(function () {
          return $q.when(job);
        })
        .catch(function (err) {
          aomLogSrv.log({ message: 'Failed to get attachments', error: err }, aomLogSrv.levels.medium, aomLogSrv.severities.err);
          return $q.reject('Failed to get attachments for job id ' + job.Id);
        });
    }

    function addJob(job) {
      return aomAPISrv.getData(job.links.self.href)
        .then(function (fullJob) {
          fullJob.isTemplate = job.isTemplate;
          job = fullJob;
          return aomCleanSrv.cleanJob(job);
        })
        .then(function () {
          return fetchAttachments(job);
        })
        .then(function () {
          return aomJobsSrv.insertJob(job, true);
        })
        .catch(function (err) {
          if (!err.jobNotAssignedToUser) {
            aomLogSrv.log({ message: 'Failed in aomJobsSrv.addJob', error: err }, aomLogSrv.levels.medium, aomLogSrv.severities.warn);
          }
        });
    }

    function processDeltaJobs(delta) {
      var promises = [];

      if (delta.added.length !== 0 || delta.deleted.length !== 0) {
        for (var i = 0; i < delta.added.length; i++) {
          promises.push(addJob(delta.added[i]));
        }
        for (var j = 0; j < delta.deleted.length; j++) {
          promises.push(deleteJob(delta.deleted[j]));
        }

        aomJobsSrv.data.canShowAnimation = promises.length < 10;

        return $q.all(promises)
          .then(function () {
            return $q.when();
          })
          .catch(function (err) {
            aomLogSrv.log({ message: 'Failed in aomJobsSrv.processDeltaJobs', error: err }, aomLogSrv.levels.medium, aomLogSrv.severities.warn);
            return $q.reject();
          });
      } else {
        return $q.when();
      }
    }

    function handleJobsResponse(response) {
      var jobsEnumerable = Enumerable.from(aomJobsSrv.data.jobs),
        templatesEnumerable = Enumerable.from(aomJobsSrv.data.templates),
        delta = {
          added: [],
          deleted: []
        };

      var deletedJobs = jobsEnumerable
        .where('$.fromTemplate !== true')
        .groupJoin(response.Jobs, '$.Id', '\'\' + $.Id', 'deviceJobs,responseJobs=>{deviceJobs:deviceJobs,responseJobs:responseJobs}')
        .selectMany(
          '$.responseJobs.defaultIfEmpty()',
          'x,y=>{Id:x.deviceJobs.Id, deleted:y!=undefined}')
        .where('$.deleted != true')
        .toArray();
      var deletedTemplates = templatesEnumerable
        .groupJoin(response.Templates, '$.Id', '\'\' + $.Id', 'deviceTemplates,responseTemplates=>{deviceTemplates:deviceTemplates,responseTemplates:responseTemplates}')
        .selectMany(
          '$.responseTemplates.defaultIfEmpty()',
          'x,y=>{Id:x.deviceTemplates.Id, deleted:y!=undefined, isTemplate:true}')
        .where('$.deleted != true')
        .toArray();

      if (deletedJobs.length > 0) {
        delta.deleted = delta.deleted.concat(deletedJobs);
      }
      if (deletedTemplates.length > 0) {
        delta.deleted = delta.deleted.concat(deletedTemplates);
      }

      for (var i = 0; i < response.Jobs.length; i++) {
        var jobsWithId = jobsEnumerable
          .where('$.Id === \'' + response.Jobs[i].Id + '\' && $.Version === ' + response.Jobs[i].Version)
          .toArray();
        if (jobsWithId.length === 0) {
          delta.added.push(response.Jobs[i]);
        }
      }
      for (var j = 0; j < response.Templates.length; j++) {
        var templateWithId = templatesEnumerable
          .where('$.Id === \'' + response.Templates[j].Id + '\' && $.Version === ' + response.Templates[j].Version)
          .toArray();
        if (templateWithId.length === 0) {
          response.Templates[j].isTemplate = true;
          delta.added.push(response.Templates[j]);
        }
      }
      return processDeltaJobs(delta);
    }

    function generateMarkers() {
      var bounds = [];

      aomJobsSrv.data.markers = {};

      var jobsForSelectedApplication = aomJobsSrv.data.filteredJobs;

      for (var i = 0; i < jobsForSelectedApplication.length; i++) {
        var element = jobsForSelectedApplication[i];
        if (element.WorkLocation !== null && element.WorkLocation.Latitude !== null && element.WorkLocation.Longitude !== null) {
          var lat = parseFloat(element.WorkLocation.Latitude),
            lng = parseFloat(element.WorkLocation.Longitude),
            address = element.WorkLocation.Address ? ' - ' + element.WorkLocation.Address : '';
          if (lat !== 0 && lng !== 0) {
            var id = 'marker' + jobsForSelectedApplication[i].Id;
            var marker = {
              group: 'Jobs',
              lat: lat,
              lng: lng,
              icon: {
                type: 'awesomeMarker',
                markerColor: aomResourcesSrv.data.resources.MapOptions.MarkerColorWorkLocation
              },
              groupOption: {
                spiderfyDistanceMultiplier: 1.5
              },
              jobId: jobsForSelectedApplication[i].Id,
              options: {
                noHide: true
              }
            };
            marker.message = '<a ng-click="edit()">' + element.Title + address + '</a>';
            marker.getMessageScope = function () {
              var scope = $rootScope.$new();
              scope.jobId = this.jobId;
              scope.edit = function () {
                aomEventSrv.trigger('OpenJob', { jobId: this.jobId });
              };
              return scope;
            };
            aomJobsSrv.data.markers[id] = marker;
            bounds.push([lat, lng]);
          }
        }
      }
      angular.copy(bounds, aomJobsSrv.data.bounds);
    };

    function filterObjects(objects, filteredList) {
      filteredList.length = 0;

      for (var i = 0; i < aomResourcesSrv.data.applicationsCopy.SelectedApplication.length; i++) {
        var tJobs = Enumerable.from(objects).where('$.ApplicationCode === "' + aomResourcesSrv.data.applicationsCopy.SelectedApplication[i].Code +
          '" && $.jsProperties.status !== "' + constants.status.deleted + '"' + ' && !$.jsProperties.submitted').toArray();
        filteredList.push.apply(filteredList, tJobs);
      }

      for (var x = 0; x < filteredList.length; x++) {
        var job = filteredList[x];
        var value = job[aomResourcesSrv.data.resources.derived.SelectedExtraInfo.Code];

        if (value === undefined) {
          value = job.WorkLocation[aomResourcesSrv.data.resources.derived.SelectedExtraInfo.Code];
        }

        if (aomResourcesSrv.data.resources.derived.SelectedExtraInfo.Filter) {
          value = $filter(aomResourcesSrv.data.resources.derived.SelectedExtraInfo.Filter)(value);
        }

        job.jsProperties.extraInfo =
          value === undefined || value === null
            ? null
            : aomResourcesSrv.data.resources.derived.SelectedExtraInfo.Code === 'Address'
              ? value
              : aomResourcesSrv.data.resources.derived.SelectedExtraInfo.Text + ': ' + value;
      }
    }

    function setupJobs() {
      filterObjects(aomJobsSrv.data.jobs, aomJobsSrv.data.filteredJobs);
      aomJobDistanceSrv.generateDistances(aomJobsSrv.data.filteredJobs);
    }

    function setupTemplates() {
      filterObjects(aomJobsSrv.data.templates, aomJobsSrv.data.filteredTemplates);
    }

    function setupCustomLocations(job) {
      var markers = Object.keys(job.markersOnJob);
      var i;
      for (i = 0; i < markers.length; i++) {
        var marker = job.markersOnJob[markers[i]];
        if (!marker.isWorkLocation) {
          job.CustomLocations.push({
            Latitude: marker.lat,
            Longitude: marker.lng,
            Comments: marker.comment,
            DateTime: marker.dateTime
          });
        }
      }
    }

    function submitJob(job) {
      return aomAttachmentsSrv.sendAttachmentsForJob(job.Id)
        .then(function () {
          return aomAttachmentsSrv.addLinksToJob(job);
        })
        .then(function () {
          var jobCopy = angular.copy(job);
          delete jobCopy.jsProperties.searchData;
          delete jobCopy.Paging;

          setupCustomLocations(jobCopy);

          delete jobCopy.markerBounds;
          delete jobCopy.markersOnJob;
          for (var i = 0; i < jobCopy.Fields.length; i++) {
            if (constants.fieldTypes.time === jobCopy.Fields[i].Type || constants.fieldTypes.date === jobCopy.Fields[i].Type) {
              if (jobCopy.Fields[i].Data === null) {
                jobCopy.Fields[i].Data = '';
              } else if (jobCopy.Fields[i].Data !== '') {
                const format = (constants.fieldTypes.date === jobCopy.Fields[i].Type) ? constants.dateFormat : constants.timeFormat;
                jobCopy.Fields[i].Data = moment(jobCopy.Fields[i].Data).format(format);
              }
            }
          }
          if (job.fromTemplate) {
            jobCopy.TransactionId = job.Id;
            jobCopy.Id = 0;
            return aomAPISrv.postData(aomAPISrv.data.rootLinks.MobileJobs.href, jobCopy);
          } else {
            return aomAPISrv.putData(job.links.self.href, jobCopy);
          }
        })
        .then(function () {
          aomLogSrv.log({ message: 'Job ' + job.Id + ' submitted', job: job }, aomLogSrv.levels.medium, aomLogSrv.severities.info);
          if (job.fromTemplate) {
            return deleteJob(job);
          } else {
            job.jsProperties.submitted = true;
          }
        })
        .catch(function (err) {
          aomLogSrv.log({ message: 'Job ' + job.Id + ' failed to submit.', job: job }, aomLogSrv.levels.medium, aomLogSrv.severities.warn);
          return $q.reject(err);
        });
    }

    var processingSetupJobs = false;
    var processSetupJobsAgain = false;

    function setupJobsWithTimeout() {
      if (processingSetupJobs) {
        processSetupJobsAgain = true;
      }
      if (setupTimeout !== null) {
        $timeout.cancel(setupTimeout);
      }
      setupTimeout = $timeout(function () {
        processingSetupJobs = true;
        setupJobs();
        setupTemplates();
        processingSetupJobs = false;
        if (processSetupJobsAgain) {
          processSetupJobsAgain = false;
          setupJobsWithTimeout();
        }
      }, 20);
    }

    function setProcessingFlagOnJobs(jobs, processing) {
      return jobs.map(function (job) {
        job.jsProperties.processing = processing;
        return job;
      });
    }


    aomJobsSrv.initJobService = function (jobId) {
      aomJobSrv.data.selectedJob = Enumerable.from(aomJobsSrv.data.jobs).where('$.Id === \'' + jobId + '\'').toArray()[0];
      aomJobSrv.data.features = {
        readonly: false,
        linkToBridge: false,
        runSql: true,
        canSubmit: true
      };
      if (aomJobSrv.data.selectedJob.jsProperties.status !== constants.status.pending && aomJobSrv.data.selectedJob.jsProperties.status !== constants.status.updating) {
        if (aomOfflineDatabasesSrv.data.importing && aomJobSrv.data.selectedJob.SqlMap.length > 0) {
          aomPopupSrv.confirm('ImportPending', {
            title: aomResourcesSrv.data.resources.Jobs.PendingImportAlertTitle,
            template: aomResourcesSrv.data.resources.Jobs.PendingImportAlertTemplate,
            onYes: function () {
              aomJobSrv.init();
            },
            onNo: function () {
            }
          });
        } else {
          aomJobSrv.init();
        }
      }
    };

    aomJobsSrv.insertJob = function (aJob, serverInitiated) {
      var job = angular.copy(aJob);

      aomCleanSrv.generateSearchData(job);

      var insertObject = {},
        inMemoryData = null,
        tableDetails = {
          id: job.Id,
          detail_json: job
        };
      if (job.isTemplate) {
        insertObject.Templates = tableDetails;
        inMemoryData = aomJobsSrv.data.templates;
      } else {
        insertObject.Jobs = tableDetails;
        inMemoryData = aomJobsSrv.data.jobs;
      }

      if (serverInitiated !== undefined && aomJobUpdatedSrv.shouldShowJobDeletedUpdatedDialog(job.Id)) {
        if (job.isTemplate) {
          return $q.when();
        } else {
          aomJobUpdatedSrv.showJobUpdatedDialog();
        }
      }

      var existing = Enumerable.from(inMemoryData).where('$.Id === \'' + job.Id + '\'').toArray();
      if (existing.length === 1 && job.Version !== existing[0].Version) {
        job.jsProperties.status = constants.status.updated;
      }

      return aomDatabaseSrv.insertQuery(insertObject, true, false)
        .then(function () {
          existing.length === 1 ? angular.copy(job, existing[0]) : inMemoryData.push(job);
          setupJobsWithTimeout();
        });
    };

    aomJobsSrv.removeFromDrafts = function removeFromDrafts(jobId) {
      return aomDatabaseSrv.deleteQuery({
        Recovery: {
          id: jobId
        }
      }).then(function () {
        return aomJobsSrv.setupDraftJobs();
      });
    };

    aomJobsSrv.changedApplicationCode = function () {
      setupJobsWithTimeout();
      generateMarkers();
    };

    aomJobSrv.validateResponse = function (response) {
      var jobs = response.Jobs.reduce(function (acc, cur) {
        acc[cur.Id] = (acc[cur.Id] || 0) + 1;
        return acc;
      }, {});

      var dupJobs = Object.keys(jobs).filter(function (obj) {
        return jobs[obj] > 1;
      }).length > 0;

      var templates = response.Templates.reduce(function (acc, cur) {
        acc[cur.Id] = (acc[cur.Id] || 0) + 1;
        return acc;
      }, {});

      var dupTemplates = Object.keys(templates).filter(function (obj) {
        return templates[obj] > 1;
      }).length > 0;

      if (dupJobs || dupTemplates) {
        aomLogSrv.log({ message: 'Duplicate Job or Template found in array', error: response }, aomLogSrv.levels.high, aomLogSrv.severities.err);
        return $q.reject();
      } else {
        return handleJobsResponse(response);
      }
    };

    aomJobsSrv.getJobs = function () {
      return aomAPISrv.getData(aomAPISrv.data.rootLinks.MobileJobs.href)
        .then(function (response) {
          if (response.Jobs) {
            $rootScope.$emit('$jobsSrv:online', true);
            aomLoginTaskManagerSrv.setRemoteAtatchments(response.Attachments);
            return handleJobsResponse(response);
          } else {
            aomLogSrv.log({ message: 'invalid object received in aomJobsSrv.getJobs.  Possibly just a reauth, response in error', error: response }, aomLogSrv.levels.medium, aomLogSrv.severities.info);
            return $q.reject();
          }
        })
        .then(function () {
          generateMarkers();
          return confirmReceipt();
        })
        .then(function () {
          aomJobsSrv.data.canShowAnimation = true;
          aomLoginTaskManagerSrv.runLoginTasks();
        })
        .catch(function (err) {
          $rootScope.$emit('$jobsSrv:online', false);
          aomLogSrv.log({ message: 'Failed in aomJobsSrv.getJobs', error: err }, aomLogSrv.levels.medium, aomLogSrv.severities.warn);
        });
    };

    aomJobsSrv.submitJobs = function () {
      var promises = [],
        jobsToSubmit = Enumerable.from(aomJobsSrv.data.jobs).where('$.readyToSubmit && $.jsProperties.processing === false && $.jsProperties.submitted === false').toArray();
      setProcessingFlagOnJobs(jobsToSubmit, true);
      for (var i = 0; i < jobsToSubmit.length; i++) {
        promises.push(submitJob(jobsToSubmit[i]));
      }
      return $q.all(promises)
        .then(function () {
          setupJobsWithTimeout();
        })
        .catch(function () {
          setProcessingFlagOnJobs(jobsToSubmit, false);
        })
        .finally(function () {
          return $q.when();
        });
    };

    aomJobsSrv.setBounds = function () {
      leafletData.getMap('map').then(function (map) {
        aomLocationSrv.getAccurateCurrentLocation()
          .then(function () {
            var currentLocationBound = [];
            currentLocationBound.push([aomLocationSrv.data.lastKnownLocation.latitude, aomLocationSrv.data.lastKnownLocation.longitude]);
            currentLocationBound.push(aomJobsSrv.data.bounds);
            map.fitBounds(currentLocationBound, { maxZoom: 16, padding: [50, 50] });
            aomJobsSrv.data.bounds.length === 0 ? map.setView(new L.LatLng(aomLocationSrv.data.lastKnownLocation.latitude, aomLocationSrv.data.lastKnownLocation.longitude), 16) : '';
          });
      });
    };

    aomJobsSrv.jobsViewLoaded = function () {
      setupJobsWithTimeout();
      aomJobSrv.data.activeJobId = null;
      aomBackSrv.clear();
    };

    aomJobsSrv.setupDraftJobs = function setupDraftJobs() {
      return aomDatabaseSrv.selectQuery({
        Recovery: {
          detail_json: null
        }
      })
        .then(function (jobs) {
          angular.copy(jobs.rows, aomJobsSrv.data.drafts);
          return $q.when();
        });
    };

    aomJobsSrv.init = function init() {
      return aomJobsSrv.setupDraftJobs()
        .then(function () {
          return aomDatabaseSrv.selectQuery({
            Jobs: {
              detail_json: null
            }
          });
        })
        .then(function (jobs) {
          aomJobsSrv.data.jobs = jobs.rows;
          return aomDatabaseSrv.selectQuery({
            Templates: {
              detail_json: null
            }
          });
        })
        .then(function (templates) {
          aomJobsSrv.data.templates = templates.rows;
        })
        .catch(function (err) {
          aomLogSrv.log({ message: 'Failed in aomJobsSrv.init', error: err }, aomLogSrv.levels.medium, aomLogSrv.severities.warn);
          return $q.reject(err);
        });
    };

    aomJobsSrv.getJobById = function getJobById(jobId, isTemplate) {
      var selectQuery = {};
      if (isTemplate) {
        selectQuery.Templates = { detail_json: null };
      } else {
        selectQuery.Jobs = { detail_json: null };
      }
      selectQuery.where = { id: jobId };
      return aomDatabaseSrv.selectQuery(selectQuery)
        .then(function (job) {
          return $q.when(job.rows);
        });
    };

    aomJobsSrv.dataForJobs = function () {
      return {
        contents: aomJobsSrv.data.filteredJobs,
        showMapIcon: true
      };
    };

    aomJobsSrv.dataForTemplates = function () {
      return {
        contents: aomJobsSrv.data.filteredTemplates,
        showMapIcon: false
      };
    };

    return aomJobsSrv;
  });