'use strict';
angular.module('aom.services')
  .factory('aomLogHandlerSrv', function ($q, $rootScope, $timeout, $cordovaFile, $http, aomResourcesSrv, aomDatabaseSrv, aomAPISrv, aomFileSystemSrv, constants) {
    var logHandlerService = {},
      fileLogs = constants.logging.fileLogs,
      logsToProcess = [],
      processing = false,
      initialised = false;

    init();

    function init() {
      return aomDatabaseSrv.selectQuery({ LogFiles: { id: null, fileName: null } })
        .then(function (fileNames) {
          if (fileNames.rows.length === 0) {
            aomFileSystemSrv.createDirectory(constants.common.logsFolder)
              .then(function () {
                initialised = setupFileLogs(fileNames);
              });
          } else {
            initialised = setupFileLogs(fileNames);
          }
        });
    }

    function setupFileLogs(fileNames) {
      var promises = [];
      for (var key in fileLogs) {
        if (fileLogs.hasOwnProperty(key)) {
          // Use an immediately executing function to have the prop value inside the closure
          (function (key) {
            var dbFileRecord = Enumerable.from(fileNames.rows).where('$.id === "' + fileLogs[key].id + '"').toArray();
            if (dbFileRecord.length === 0) {
              fileLogs[key].currentFile = 'B';
              promises.push(
                aomFileSystemSrv.createDirectory(fileLogs[key].dir)
                  .then(function () {
                    return rollFile(fileLogs[key]);
                  })
              );
            } else {
              fileLogs[key].currentFile = dbFileRecord[0].fileName;
              promises.push(rollFile(fileLogs[key]));
            }
          }(key));
        }
      }
      return $q.all(promises);
    }

    function logToDB(log) {
      log.object.error = null;
      var query = {
        Logs: {
          log_json: log,
          severity: log.severity,
          timestamp: log.time
        }
      };
      aomDatabaseSrv.insertQuery(query, false, false);
    }

    function removeDBLogs() {
      return aomDatabaseSrv.selectQuery({ Logs: { id: null } })
        .then(function (logIds) {
          if (logIds.rows.length !== 0) {
            var maxId = Math.max.apply(null, logIds.rows);
            return aomDatabaseSrv.deleteQuery({ Logs: { id: ';< ' + (maxId - constants.logging.dbLogsToKeep) } });
          }
        });
    }

    function rollFile(fileLog) {
      fileLog.currentFile = fileLog.currentFile === 'A' ? 'B' : 'A';
      var query = { LogFiles: { id: fileLog.id, fileName: fileLog.currentFile } };
      return aomDatabaseSrv.insertQuery(query, true, false)
        .then(function () {
          fileLog.count = 0;
          return aomFileSystemSrv.write(fileLog.currentFile + '.txt', { data: 'Cleared File ' + moment().format() }, fileLog.dir, false);
        });
    }

    function getLogName(fileLog) {
      fileLog.count++;
      if (fileLog.count > fileLog.minRecords) {
        return rollFile(fileLog);
      }
      return $q.when();
    }

    function logToFile(log, fileLog) {
      return getLogName(fileLog)
        .then(function () {
          return aomFileSystemSrv.write(fileLog.currentFile + '.txt', { data: JSON.stringify(log) }, fileLog.dir, true);
        });
    }

    function logToConsole(log) {
      if (constants.logging.logToConsole) {
        console.log(log);
      }
    }

    function processLog(log) {
      switch (log.level) {
        case constants.logging.levels.user:
          logToDB(log);
          return logToFile(log, fileLogs.high);
        case constants.logging.levels.high:
          return logToFile(log, fileLogs.high);
        case constants.logging.levels.medium:
          return logToFile(log, fileLogs.medium);
        default:
          // TODO Revert default to do nothing
          // return $q.when();
          return logToFile(log, fileLogs.medium);
      }
    }

    function processLogs() {
      processing = true;
      if (initialised) {
        if (logsToProcess.length === 0) {
          processing = false;
        } else {
          processLog(logsToProcess.shift())
            .finally(function () {
              $timeout(function () {
                processLogs();
              });
            });
        }
      }
    }

    function log(event, log) {
      event.stopPropagation();
      if (constants.logging.levels[aomResourcesSrv.data.resources.LogLevel] <= log.level) {
        $rootScope.$emit('slackLog', log);      
        logsToProcess.push(log);
        logToConsole(log);
        if (!processing) {
          processLogs();
        }
      }
    }

    $rootScope.$on('aomLog', log);

    logHandlerService.housekeep = function () {
      return removeDBLogs();
    };

    logHandlerService.getLogs = function () {
      return aomDatabaseSrv.selectQuery({ Logs: { log_json: null } });
    };
    return logHandlerService;
  });