'use strict';
angular.module('aom.directives')
  .directive('aomMap', function ($state, $timeout, $ionicActionSheet, aomJobsSrv, leafletMarkersHelpers, constants, aomResourcesSrv, leafletData, aomJobSrv, aomLocationSrv, aomComponentsSrv, aomNetworkSrv, aomStateSrv, aomBackSrv) {
    return {
      scope: {
        onSelect: '&'
      },
      controller: function ($scope) {
        $scope.data = {
          jobServiceData: aomJobsSrv.data,
          jobs: aomJobSrv.data.jobs,
          selectedJobDetails: {},
          resources: aomResourcesSrv.data.resources,
          showMapNavigation: false
        };

        var jobsMap = null;

        var ionicBeforeEnter = $scope.$on('$ionicView.beforeEnter', function (event, viewData) {
          viewData.enableBack = false;
        });

        aomJobsSrv.jobsViewLoaded();

        window.addEventListener('orientationchange', function () {
          $timeout(function () {
            jobsMap.invalidateSize();
            jobsMap._resetView(jobsMap.getCenter(), jobsMap.getZoom(), true);
          }, 500);
        }, false);

        this.$onInit = function () {
          aomBackSrv.init($scope.backClicked);
        };

        $scope.backClicked = function backClicked() {
          aomStateSrv.setRight(constants.viewStates.empty);
        };

        $scope.showList = function () {
          $timeout(function () {
            $state.go('app.Jobs');
          }, 30);
        };

        angular.extend(
          $scope, {
            defaults: {
              maxZoom: 19,
              minZoom: 3,
              zoomAnimation: true
            },
            layers: {
              baselayers: {}
            },
            markers: $scope.data.jobServiceData.markers
          });

        $scope.$on('$destroy', function () {
          ionicBeforeEnter();
          leafletMarkersHelpers.resetMarkerGroups();
        });

        $scope.init = function () {
          leafletData.getMap('map')
            .then(function (map) {
              jobsMap = map;

              var currentLocationControl = aomComponentsSrv.getLeafletCurrentLocationControl(function () {
                aomLocationSrv.getCurrentLocation()
                  .then(function () {
                    var location = aomLocationSrv.data.lastKnownLocation;
                    jobsMap.panTo([location.latitude, location.longitude]);
                  });
              });

              map.addControl(currentLocationControl);
              map.worldCopyJump = false;
              map.removeControl(map.attributionControl);

              $timeout(function () {
                jobsMap.invalidateSize();
                $scope.layers.baselayers = {
                  googleTerrain: {
                    top: aomResourcesSrv.data.resources.derived.MapViewSelected.Code === 'TERRAIN',
                    name: aomResourcesSrv.data.resources.MapOptions.Terrain,
                    layerType: 'TERRAIN',
                    type: 'google'
                  },
                  googleHybrid: {
                    top: aomResourcesSrv.data.resources.derived.MapViewSelected.Code === 'HYBRID',
                    name: aomResourcesSrv.data.resources.MapOptions.Hybrid,
                    layerType: 'HYBRID',
                    type: 'google'
                  },
                  googleRoadmap: {
                    top: aomResourcesSrv.data.resources.derived.MapViewSelected.Code === 'ROADMAP',
                    name: aomResourcesSrv.data.resources.MapOptions.Road,
                    layerType: 'ROADMAP',
                    type: 'google'
                  }
                };
                L.marker(
                  [aomLocationSrv.data.lastKnownLocation.latitude, aomLocationSrv.data.lastKnownLocation.longitude],
                  { icon: aomComponentsSrv.markerColourCurrentLocation, zIndexOffset: -9000 }
                ).addTo(jobsMap).bindPopup(aomResourcesSrv.data.resources.MapOptions.MarkerPopupCurrentLocation);
                jobsMap.setMaxBounds([
                  [-90, -180],
                  [90, 180]
                ]);
                aomJobsSrv.setBounds();
              }, 300);
            });
        };

        $scope.$on('leafletDirectiveMarker.map.click', function (event, args) {
          var selectedJob = Enumerable.from(aomJobsSrv.data.jobs).where('$.Id === "' + args.model.jobId + '"').toArray()[0];
          if (selectedJob) {
            $scope.data.showMapNavigation = true;
            $scope.data.selectedJobDetails = { title: selectedJob.Title, workLocation: selectedJob.WorkLocation, jobId: selectedJob.Id };
          }
          $scope.onSelect()({ jobId: args.model.jobId });
        });

        $scope.showOptions = function () {

          var options = [
            { text: $scope.data.resources.AomActionSheetOptions.CurrentLocationPan, key: 'CurrentLocation' }
          ];

          $ionicActionSheet.show({
            buttons: options,
            titleText: $scope.data.resources.AomActionSheetOptions.Title,
            buttonClicked: function (index, object) {
              if (object.key === 'CurrentLocation') {
                aomLocationSrv.getCurrentLocation()
                  .then(function () {
                    jobsMap.panTo([aomLocationSrv.data.lastKnownLocation.latitude, aomLocationSrv.data.lastKnownLocation.longitude]);
                  });
              }
              return true;
            }
          });
        };
      },
      templateUrl: 'App/Components/Map/map.html'
    };
  });
